#!/bin/bash
#
#  IP Accounting v 1.2
#  Tento program slouzi ke statistice sitoveho provozu pro jednotliva IP
#  Created by standus - standus@standus.com

active_iface="eth0 eth1"
dir_data_txt="/var/www/html/ipaccounting"
dir_data_rdd="/var/www/html/ipaccounting/rrd"
rdd_active="on"
txt_active="on"
testing_active="on"
verbose="on"

# Testovani pravidel a chainu v IPTABLES
if [ $testing_active == "on" ]; then
    if [ "`iptables -t mangle -L POSTROUTING | grep "DATA_IN"`" == "" ]; then
        if [ $verbose == "on" ]; then
	 echo "Pridavam chain DATA_IN"
	fi
        iptables -t mangle -N DATA_IN
        iptables -t mangle -A POSTROUTING -j DATA_IN
    fi
    if [ "`iptables -t mangle -L PREROUTING | grep "DATA_OUT"`" == "" ]; then
	if [ $verbose == "on" ]; then
	 echo "Pridavam chain DATA_OUT"
	fi
        iptables -t mangle -N DATA_OUT
        iptables -t mangle -A PREROUTING -j DATA_OUT
    fi
    
    for iface in $active_iface; do
	for ip in `cat /proc/net/arp | grep "$iface" | awk '{print $1}'`; do
    	    if [ $verbose == "on" ]; then
	     echo "Testuji ip: $ip"
	    fi 
    	    if [ "`iptables -t mangle -L DATA_IN  -v -x -n | grep " $ip " | awk '{print $2}'`" == "" ]; then
		if [ $verbose == "on" ]; then
		 echo "Pridavam pravidlo DATA_IN pro ip: $ip"
		fi
		iptables -t mangle -A DATA_IN -d $ip -j RETURN
	    fi
	    if [ "`iptables -t mangle -L DATA_OUT  -v -x -n | grep " $ip " | awk '{print $2}'`" == "" ]; then
		if [ $verbose == "on" ]; then
		 echo "Pridavam pravidlo DATA_OUT pro ip: $ip"
		fi 
		iptables -t mangle -A DATA_OUT -s $ip -j RETURN
	    fi
	done
    done
fi

# Otevreni textaku s daty
counter_file=`cat $dir_data_txt/data.txt`
counter_file_d=`cat $dir_data_txt/data-daily.txt`
counter_file_w=`cat $dir_data_txt/data-weekly.txt`
counter_file_m=`cat $dir_data_txt/data-monthly.txt`
counter_file_y=`cat $dir_data_txt/data-yearly.txt`
# Nacteni dat z IPTABLES
data=`iptables -t mangle -L DATA_OUT  -v -x -n -Z  | sed -e 's/ /-/g' | grep "RETURN"`
# Statistika v textaku od pocatku pripojeni pocitace
for i in $data; do
    ip=`echo $i | sed -e 's/-/ /g' | awk '{print $7}'`
    out=`echo $i | sed -e 's/-/ /g' | awk '{print $2}'`
    out_old=`echo -ne "$counter_file"  | grep "$ip-" | sed -e 's/-/ /g' | awk '{print $2}'`
    out_old_d=`echo -ne "$counter_file_d"  | grep "$ip-" | sed -e 's/-/ /g' | awk '{print $2}'`
    out_old_w=`echo -ne "$counter_file_w"  | grep "$ip-" | sed -e 's/-/ /g' | awk '{print $2}'`
    out_old_m=`echo -ne "$counter_file_m"  | grep "$ip-" | sed -e 's/-/ /g' | awk '{print $2}'`
    out_old_y=`echo -ne "$counter_file_y"  | grep "$ip-" | sed -e 's/-/ /g' | awk '{print $2}'`
    out_new=$(($out_old + $out))
    out_new_d=$(($out_old_d + $out))
    out_new_w=$(($out_old_w + $out))
    out_new_m=$(($out_old_m + $out))
    out_new_y=$(($out_old_y + $out))
    new_file_out="$new_file_out$ip-$out_new\n"
    new_file_out_d="$new_file_out_d$ip-$out_new_d\n"
    new_file_out_w="$new_file_out_w$ip-$out_new_w\n"
    new_file_out_m="$new_file_out_m$ip-$out_new_m\n"
    new_file_out_y="$new_file_out_y$ip-$out_new_y\n"
    out_aver=$(($out / 300))
    new_rdd_out="$new_rdd_out$ip-$out_aver\n"
    if [ $verbose == "on" ]; then
     echo -ne "$ip\tOUT\tstare: $out_old \tnove: $out \tsoucet: $out_new\trate: $out_aver\n"
    fi
done
## IN_DATA /-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/

data=`iptables -t mangle -L DATA_IN  -v -x -n -Z  | sed -e 's/ /-/g' | grep "RETURN"`
for i in $data; do
    ip=`echo $i | sed -e 's/-/ /g' | awk '{print $8}'`
    in=`echo $i | sed -e 's/-/ /g' | awk '{print $2}'`
    in_old=`echo -ne "$counter_file"  | grep "$ip-" | sed -e 's/-/ /g' | awk '{print $3}'`
    in_old_d=`echo -ne "$counter_file_d"  | grep "$ip-" | sed -e 's/-/ /g' | awk '{print $3}'`
    in_old_w=`echo -ne "$counter_file_w"  | grep "$ip-" | sed -e 's/-/ /g' | awk '{print $3}'`
    in_old_m=`echo -ne "$counter_file_m"  | grep "$ip-" | sed -e 's/-/ /g' | awk '{print $3}'`
    in_old_y=`echo -ne "$counter_file_y"  | grep "$ip-" | sed -e 's/-/ /g' | awk '{print $3}'`
    in_new=$(($in_old + $in))
    in_new_d=$(($in_old_d + $in))
    in_new_w=$(($in_old_w + $in))
    in_new_m=$(($in_old_m + $in))
    in_new_y=$(($in_old_y + $in))
    new_file_in="$new_file_in$ip-$in_new\n"
    new_file_in_d="$new_file_in_d$ip-$in_new_d\n"
    new_file_in_w="$new_file_in_w$ip-$in_new_w\n"
    new_file_in_m="$new_file_in_m$ip-$in_new_m\n"
    new_file_in_y="$new_file_in_y$ip-$in_new_y\n"
    in_aver=$(($in / 300))
    new_rdd_in="$new_rdd_in$ip-$in_aver\n"
    if [ $verbose == "on" ]; then
     echo -ne "$ip\tIN\tstare: $in_old \tnove: $in \tsoucet: $in_new\trate: $in_aver\n"
    fi
done
### Zapsani hodnot do txt souboru (pocitadlo prenesenych dat)
if [ $txt_active == "on" ]; then
    for i in `echo -ne $new_file_out`; do
	ip=`echo $i | sed -e 's/-/ /g' | awk '{print $1}'`
	out=`echo $i | sed -e 's/-/ /g' | awk '{print $2}'`
	out_d=`echo -ne "$new_file_out_d" | grep "$ip-" | sed -e 's/-/ /g' | awk '{print $2}'`
	out_w=`echo -ne "$new_file_out_w" | grep "$ip-" | sed -e 's/-/ /g' | awk '{print $2}'`
	out_m=`echo -ne "$new_file_out_m" | grep "$ip-" | sed -e 's/-/ /g' | awk '{print $2}'`
	out_y=`echo -ne "$new_file_out_y" | grep "$ip-" | sed -e 's/-/ /g' | awk '{print $2}'`
	in=`echo -ne "$new_file_in" | grep "$ip-" | sed -e 's/-/ /g' | awk '{print $2}'`
	in_d=`echo -ne "$new_file_in_d" | grep "$ip-" | sed -e 's/-/ /g' | awk '{print $2}'`
	in_w=`echo -ne "$new_file_in_w" | grep "$ip-" | sed -e 's/-/ /g' | awk '{print $2}'`
	in_m=`echo -ne "$new_file_in_m" | grep "$ip-" | sed -e 's/-/ /g' | awk '{print $2}'`
	in_y=`echo -ne "$new_file_in_y" | grep "$ip-" | sed -e 's/-/ /g' | awk '{print $2}'`
	counter_data="$counter_data$ip-$out-$in\n"
	counter_data_d="$counter_data_d$ip-$out_d-$in_d\n"
	counter_data_w="$counter_data_w$ip-$out_w-$in_w\n"
	counter_data_m="$counter_data_m$ip-$out_m-$in_m\n"
	counter_data_y="$counter_data_y$ip-$out_y-$in_y\n"
    done
    echo -ne $counter_data > $dir_data_txt/data.txt
    echo -ne $counter_data_d > $dir_data_txt/data-daily.txt
    echo -ne $counter_data_w > $dir_data_txt/data-weekly.txt
    echo -ne $counter_data_m > $dir_data_txt/data-monthly.txt
    echo -ne $counter_data_y > $dir_data_txt/data-yearly.txt
fi
# nulovani pocitadel
if [ ! -e "$dir_data_txt/data-daily.lock" ]; then
    if [ `date +%H` -eq 00 ]; then
       echo > $dir_data_txt/data-daily.txt
       echo > $dir_data_txt/data-daily.lock
    fi
elif [ `date +%H` -ne 00 ]; then
       rm -f $dir_data_txt/data-daily.lock
fi
######################
if [ ! -e "$dir_data_txt/data-weekly.lock" ]; then
    if [ `date +%w` -eq 1 ]; then
       echo > $dir_data_txt/data-weekly.txt
       echo > $dir_data_txt/data-weekly.lock
    fi
elif [ `date +%w` -ne 1 ]; then 
       rm -f $dir_data_txt/data-weekly.lock
fi
#######################
if [ ! -e "$dir_data_txt/data-monthly.lock" ]; then
    if [ `date +%d` -eq 01 ]; then
       echo > $dir_data_txt/data-monthly.txt
       echo > $dir_data_txt/data-monthly.lock
    fi
elif [ `date +%d` -ne 01 ]; then 
       rm -f $dir_data_txt/data-monthly.lock
fi    
#######################
if [ ! -e "$dir_data_txt/data-yearly.lock" ]; then
    if [ `date +%j` -eq 001 ]; then
       echo > $dir_data_txt/data-yearly.txt
       echo > $dir_data_txt/data-yearly.lock
    fi
elif [ `date +%j` -ne 001 ]; then 
       rm -f $dir_data_txt/data-yearly.lock
fi    
# Zapsani hodnot do rrd databaze (grafy prenosu)
if [ $rdd_active == "on" ]; then
    for i in `echo -ne $new_rdd_out`; do
        ip=`echo $i | sed -e 's/-/ /g' | awk '{print $1}'`
	out=`echo $i | sed -e 's/-/ /g' | awk '{print $2}'`
        in=`echo -ne "$new_rdd_in" | grep "$ip-" | sed -e 's/-/ /g' | awk '{print $2}'`
	    if [ ! -e "$dir_data_rdd/host-$ip.rrd" ]; then
		rrdtool create "$dir_data_rdd/host-$ip.rrd" --step 300 DS:in:GAUGE:600:0:U DS:out:GAUGE:600:0:U RRA:AVERAGE:0.5:1:290 RRA:AVERAGE:0.5:2:1010 RRA:AVERAGE:0.5:6:1540 RRA:AVERAGE:0.5:288:370;
	    fi
	    rrdtool update "$dir_data_rdd/host-$ip.rrd" -t in:out N:$in:$out    
    done
fi
